import 'package:flutter/material.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

/// Possible shapes for [SeniorBadge].
enum SeniorBadgeShape { pill, chip }

class SeniorBadge extends StatelessWidget {
  /// Creates a badge component.
  ///
  /// [backgroundColor], [fontColor] and [label] parameters need to be provided.
  /// If [disabledBackgroundColor] and [disabledFontColor] parameters are not
  /// specified, the values of [backgroundColor] and [fontColor] parameters
  /// respectively will be used.
  const SeniorBadge({
    Key? key,
    required this.backgroundColor,
    this.disabled = false,
    this.disabledBackgroundColor,
    this.disabledFontColor,
    required this.fontColor,
    required this.label,
    this.outlined = false,
    this.shape = SeniorBadgeShape.chip,
    this.iconColor,
    this.disabledIconColor,
  }) : super(key: key);

  /// Creates a variation of [SeniorBadge] with an icon preceding the badge
  /// text.
  ///
  /// The [icon] parameter is required.
  factory SeniorBadge.icon({
    Key? key,
    required Color backgroundColor,
    bool disabled,
    Color? disabledBackgroundColor,
    Color? disabledFontColor,
    required Color fontColor,
    required IconData icon,
    required String label,
    bool outlined,
    SeniorBadgeShape shape,
    Color? iconColor,
    Color? disabledIconColor,
  }) = _SeniorBadgeIcon;

  /// The background color of the badge.
  final Color backgroundColor;

  /// The background color displayed when the badge is disabled.
  ///
  /// If this parameter is not specified, the value of [backgroundColor] will
  /// be used.
  final Color? disabledBackgroundColor;

  /// The font color of the badge.
  final Color fontColor;

  /// The icon color of the badge.
  /// If null the value of [fontColor] will be used.
  final Color? iconColor;

  /// The icon color displayed when the badge is disabled.
  /// If null the value of [disabledFontColor] will be used.
  final Color? disabledIconColor;

  /// The font color displayed when the badge is disabled.
  ///
  /// If this parameter is not specified, the value of [fontColor] will be used.
  final Color? disabledFontColor;

  /// The badge text.
  final String label;

  /// Whether the badge will be disabled or not.
  ///
  /// By default it is false, enabled.
  final bool disabled;

  /// The shape of the badge.
  ///
  /// Possible values are [SeniorBadgeShape.chip] and [SeniorBadgeShape.pill].
  final SeniorBadgeShape shape;

  /// Defines if de component has border
  final bool outlined;

  Color _getBackgroundColor() {
    if (disabledBackgroundColor == null) {
      return backgroundColor;
    }
    return disabled ? disabledBackgroundColor! : backgroundColor;
  }

  Color _getFontColor() {
    if (disabledFontColor == null) {
      return fontColor;
    }
    return disabled ? disabledFontColor ?? fontColor : fontColor;
  }

  Color _getIconColor() {
    if (iconColor == null) {
      return fontColor;
    }
    return disabled ? disabledIconColor ?? iconColor! : iconColor!;
  }

  double _getShapeRadius() {
    switch (shape) {
      case SeniorBadgeShape.pill:
        return SeniorRadius.huge;
      case SeniorBadgeShape.chip:
      default:
        return SeniorRadius.medium;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(
        horizontal: SeniorSpacing.normal,
        vertical: SeniorSpacing.xxsmall,
      ),
      decoration: BoxDecoration(
        border: outlined ? Border.all(color: SeniorColors.secondaryColor200, width: 1) : null,
        color: _getBackgroundColor(),
        borderRadius: BorderRadius.all(
          Radius.circular(_getShapeRadius()),
        ),
      ),
      child: Text(
        label,
        style: SeniorTypography.small(
          color: _getFontColor(),
        ),
      ),
    );
  }
}

class _SeniorBadgeIcon extends SeniorBadge {
  const _SeniorBadgeIcon({
    Key? key,
    required Color backgroundColor,
    bool disabled = false,
    Color? disabledBackgroundColor,
    Color? disabledFontColor,
    required Color fontColor,
    required this.icon,
    required String label,
    bool outlined = false,
    SeniorBadgeShape shape = SeniorBadgeShape.pill,
    Color? iconColor,
    Color? disabledIconColor,
  }) : super(
          key: key,
          backgroundColor: backgroundColor,
          disabled: disabled,
          disabledBackgroundColor: disabledBackgroundColor,
          disabledFontColor: disabledFontColor,
          fontColor: fontColor,
          label: label,
          shape: shape,
          outlined: outlined,
          iconColor: iconColor,
          disabledIconColor: disabledIconColor,
        );

  /// The Badge Icon.
  final IconData icon;

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(
        vertical: SeniorSpacing.xxsmall,
        horizontal: SeniorSpacing.small,
      ),
      decoration: BoxDecoration(
        border: outlined ? Border.all(color: SeniorColors.secondaryColor200, width: 1) : null,
        color: _getBackgroundColor(),
        borderRadius: BorderRadius.all(
          Radius.circular(_getShapeRadius()),
        ),
      ),
      child: Row(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Padding(
            padding: const EdgeInsets.only(
              right: SeniorSpacing.xsmall,
            ),
            child: Icon(
              icon,
              color: _getIconColor(),
              size: 14.0,
            ),
          ),
          Text(
            label,
            style: SeniorTypography.small(
              color: _getFontColor(),
            ),
          ),
        ],
      ),
    );
  }
}
